@@ -4,11 +4,10 @@ package Data::Edit::Struct;
 
 use strict;
 use warnings;
-use experimental qw[ postderef switch signatures ];
 
 use Exporter 'import';
 
-our $VERSION = '0.03'; # TRIAL
+our $VERSION = '0.04'; # TRIAL
 
 use Ref::Util qw[
   is_plain_arrayref is_arrayref
@@ -54,7 +53,9 @@ my %source = (
     spath => { type => Str,       optional => 1 },
     stype => { type => UseDataAs, default  => 'auto' },
     sxfrm => {
-        type => Enum [ 'iterate', 'array', 'hash', 'error' ] | CodeRef,
+	      # ( Enum [] ) | CoderRef rather than Enum[] | CodeRef for
+	      # Perl < 5.14
+        type => ( Enum [ 'iterate', 'array', 'hash', 'error' ] ) | CodeRef,
         default => 'error'
     },
     sxfrm_args => {
@@ -99,21 +100,26 @@ my %Validation = (
     },
 );
 
-my %Validator
-  = map { $_ => validation_for( params => $Validation{$_},
-                                name => $_,
-                                name_is_optional => 1,
-                              ) }
+my %Validator = map {
+    $_ => validation_for(
+        params           => $Validation{$_},
+        name             => $_,
+        name_is_optional => 1,
+      )
+  }
   keys %Validation;
 
-sub _dup_context ( $context ) {
+sub _dup_context {
+
+    my ( $context ) = @_;
+
     Data::DPath::Context->new( give_references => 1 )
       ->current_points( $context->current_points );
 }
 
+sub edit {
 
-
-sub edit ( $action, $params ) {
+    my ( $action, $params ) = @_;
 
     Data::Edit::Struct::failure::input::param->throw( "no action specified\n" )
       unless defined $action;
@@ -130,61 +136,61 @@ sub edit ( $action, $params ) {
       = _dup_context( $arg{dest} )->_search( dpathr( $arg{dpath} ) )
       ->current_points;
 
-    for ( $action ) {
-
-        when ( 'pop' ) {
-            _pop( $points, $arg{length} );
-        }
-
-        when ( 'shift' ) {
-
-            _shift( $points, $arg{length} );
-
-        }
-
-        when ( 'splice' ) {
-
-            $src //= [ \[] ];
-
-            _splice( $arg{dtype}, $points, $arg{offset}, $arg{length},
-                _deref( $_, $arg{stype}, $arg{clone} ) )
-              foreach @$src;
-        }
-
-        when ( 'insert' ) {
-            Data::Edit::Struct::failure::input::src->throw(
-                "source was not specified" )
-              if !defined $src;
-
-            _insert( $arg{dtype}, $points, $arg{insert}, $arg{anchor},
-                $arg{pad}, $arg{offset},
-                _deref( $_, $arg{stype}, $arg{clone} ) )
-              foreach @$src;
-        }
-
-
-        when ( 'delete' ) {
-            _delete( $points, $arg{length} );
-        }
-
-        when ( 'replace' ) {
-
-            Data::Edit::Struct::failure::input::src->throw(
-                "source was not specified" )
-              if !defined $src;
-
-            Data::Edit::Struct::failure::input::src->throw(
-                "source path may not have multiple resolutions" )
-              if @$src > 1;
-
-            _replace( $points, $arg{replace}, $src->[0] );
-        }
+    if ( $action eq 'pop' ) {
+        _pop( $points, $arg{length} );
     }
 
+    elsif ( $action eq 'shift' ) {
+        _shift( $points, $arg{length} );
+    }
+
+    elsif ( $action eq 'splice' ) {
+
+        $src = [ \[] ] if ! defined $src;
+
+        _splice( $arg{dtype}, $points, $arg{offset}, $arg{length},
+            _deref( $_, $arg{stype}, $arg{clone} ) )
+          foreach @$src;
+    }
+
+    elsif ( $action eq 'insert' ) {
+        Data::Edit::Struct::failure::input::src->throw(
+            "source was not specified" )
+          if !defined $src;
+
+        _insert( $arg{dtype}, $points, $arg{insert}, $arg{anchor},
+            $arg{pad}, $arg{offset}, _deref( $_, $arg{stype}, $arg{clone} ) )
+          foreach @$src;
+    }
+
+
+    elsif ( $action eq 'delete' ) {
+        _delete( $points, $arg{length} );
+    }
+
+    elsif ( $action eq 'replace' ) {
+
+        Data::Edit::Struct::failure::input::src->throw(
+            "source was not specified" )
+          if !defined $src;
+
+        Data::Edit::Struct::failure::input::src->throw(
+            "source path may not have multiple resolutions" )
+          if @$src > 1;
+
+        _replace( $points, $arg{replace}, $src->[0] );
+    }
+
+    else {
+        Data::Edit::Struct::failure::internal->throw(
+            "unexpected action: $action" );
+    }
 }
 
 
-sub _sxfrm ( $src, $spath, $sxfrm, $args ) {
+sub _sxfrm {
+
+    my ( $src, $spath, $sxfrm, $args ) = @_;
 
     return unless defined $src;
 
@@ -214,103 +220,97 @@ sub _sxfrm ( $src, $spath, $sxfrm, $args ) {
 
     $spath = dpath( $spath );
 
-    for ( $sxfrm ) {
+    if ( is_coderef( $sxfrm ) ) {
+        return $sxfrm->( $ctx, $spath, $args );
+    }
 
-        when ( !!is_coderef( $_ ) ) {
+    elsif ( $sxfrm eq 'array' ) {
+        $ctx->give_references( 0 );
+        return [ \$ctx->matchr( $spath ) ];
+    }
 
-            return $_->( $ctx, $spath, $args );
-        }
+    elsif ( $sxfrm eq 'hash' ) {
 
-        when ( 'array' ) {
-            $ctx->give_references( 0 );
-            return [ \$ctx->matchr( $spath ) ];
-        }
+        my %src;
 
-        when ( 'hash' ) {
-
-            my %src;
-
-            if ( exists $args->{key} ) {
-
-                my $src = $ctx->matchr( $spath );
-                Data::Edit::Struct::failure::input::src->throw(
-                    "source path may not have multiple resolutions\n" )
-                  if @$src > 1;
-                $src{ $args->{key} } = $src->[0]->$*;
-            }
-
-            else {
-
-                $ctx->give_references( 0 );
-                for my $point ( $ctx->_search( $spath )->current_points->@* ) {
-
-                    my $attrs = $point->attrs;
-                    defined( my $key = $attrs->{key} // $attrs->{idx} )
-                      or Data::Edit::Struct::failure::input::src->throw(
-                        "source path returned multiple values; unable to convert into hash as element has no `key' or `idx' attribute\n"
-                      );
-                    $src{$key} = $point->ref->$*;
-                }
-            }
-
-            return [ \\%src ];
-        }
-
-        when ( 'iterate' ) {
-
-            return $ctx->matchr( $spath );
-
-        }
-
-        default {
+        if ( exists $args->{key} ) {
 
             my $src = $ctx->matchr( $spath );
             Data::Edit::Struct::failure::input::src->throw(
                 "source path may not have multiple resolutions\n" )
               if @$src > 1;
-
-            return $src;
+            $src{ $args->{key} } = ${ $src->[0] };
         }
 
+        else {
+
+            $ctx->give_references( 0 );
+            for my $point ( @{ $ctx->_search( $spath )->current_points } ) {
+
+                my $attrs = $point->attrs;
+                defined( my $key = defined $attrs->{key} ? $attrs->{key} : $attrs->{idx} )
+                  or Data::Edit::Struct::failure::input::src->throw(
+                    "source path returned multiple values; unable to convert into hash as element has no `key' or `idx' attribute\n"
+                  );
+                $src{$key} = ${ $point->ref };
+            }
+        }
+
+        return [ \\%src ];
+    }
+
+    elsif ( $sxfrm eq 'iterate' ) {
+
+        return $ctx->matchr( $spath );
+
+    }
+
+    else {
+
+        my $src = $ctx->matchr( $spath );
+        Data::Edit::Struct::failure::input::src->throw(
+            "source path may not have multiple resolutions\n" )
+          if @$src > 1;
+
+        return $src;
     }
 }
 
 
-sub _clone ( $ref ) {
+sub _clone {
+
+    my ( $ref ) = @_;
 
     require Storable;
 
     return Storable::dclone( $ref );
 }
 
-sub _deref ( $ref, $stype, $clone ) {
+sub _deref {
+
+    my ( $ref, $stype, $clone ) = @_;
 
     $stype = is_plain_arrayref( $$ref )
       || is_plain_hashref( $$ref ) ? 'container' : 'element'
       if $stype eq 'auto';
 
     my $struct;
-    for ( $stype ) {
+    if ( $stype eq 'element' ) {
+        $struct = [$$ref];
+    }
 
-        when ( 'element' ) {
-            $struct = [$$ref];
-        }
+    elsif ( $stype eq 'container' ) {
 
-        when ( 'container' ) {
-
-            $struct
-              = is_arrayref( $$ref ) ? $$ref
-              : is_hashref( $$ref )  ? [%$$ref]
-              : Data::Edit::Struct::failure::input::src->throw(
-                "\$value is not an array or hash reference" );
-        }
-
-        default {
-
-            Data::Edit::Struct::failure::internal->throw(
-                "internal error: unknown mode to use source in: $_" );
-        }
+        $struct
+          = is_arrayref( $$ref ) ? $$ref
+          : is_hashref( $$ref )  ? [%$$ref]
+          : Data::Edit::Struct::failure::input::src->throw(
+            "\$value is not an array or hash reference" );
+    }
 
+    else {
+        Data::Edit::Struct::failure::internal->throw(
+            "internal error: unknown mode to use source in: $_" );
     }
 
     $clone = \&_clone unless is_coderef( $clone ) || !$clone;
@@ -319,14 +319,15 @@ sub _deref ( $ref, $stype, $clone ) {
         is_coderef( $clone ) ? $clone->( $struct )
       : $clone               ? _clone( $struct )
       :                        $struct;
-
 }
 
-sub _pop ( $points, $length ) {
+sub _pop {
+
+    my ( $points, $length ) = @_;
 
     for my $point ( @$points ) {
 
-        my $dest = $point->ref->$*;
+        my $dest = ${ $point->ref };
         Data::Edit::Struct::failure::input::dest->throw(
             "destination is not an array" )
           unless is_arrayref( $dest );
@@ -337,10 +338,12 @@ sub _pop ( $points, $length ) {
     }
 }
 
-sub _shift ( $points, $length ) {
+sub _shift {
+
+    my ( $points, $length ) = @_;
 
     for my $point ( @$points ) {
-        my $dest = $point->ref->$*;
+        my $dest = ${ $point->ref };
         Data::Edit::Struct::failure::input::dest->throw(
             "destination is not an array" )
           unless is_arrayref( $dest );
@@ -348,7 +351,9 @@ sub _shift ( $points, $length ) {
     }
 }
 
-sub _splice ( $dtype, $points, $offset, $length, $replace ) {
+sub _splice {
+
+    my ( $dtype, $points, $offset, $length, $replace ) = @_;
 
     for my $point ( @$points ) {
 
@@ -356,7 +361,7 @@ sub _splice ( $dtype, $points, $offset, $length, $replace ) {
 
         my $attrs = $point->can( 'attrs' );
 
-        my $idx = ( ( defined( $attrs ) && $point->$attrs ) // {} )->{idx};
+        my $idx = ( ( defined( $attrs ) && $point->$attrs ) ? $point->$attrs : {} )->{idx};
 
         my $use = $dtype;
 
@@ -371,37 +376,41 @@ sub _splice ( $dtype, $points, $offset, $length, $replace ) {
                 "point is neither an array element nor an array ref" );
         }
 
-        for ( $use ) {
+        if ( $use eq 'container' ) {
+            $ref = $point->ref if ! defined $ref;
+            Data::Edit::Struct::failure::input::dest->throw(
+                "point is not an array reference" )
+              unless is_arrayref( $$ref );
 
-            when ( 'container' ) {
-                $ref //= $point->ref;
-                Data::Edit::Struct::failure::input::dest->throw(
-                    "point is not an array reference" )
-                  unless is_arrayref( $$ref );
+            splice( @{ $$ref }, $offset, $length, @$replace );
+        }
 
-                splice( $$ref->@*, $offset, $length, @$replace );
-            }
+        elsif ( $use eq 'element' ) {
 
-            when ( 'element' ) {
+            my $rparent = $point->parent;
+            my $parent
+              = defined( $rparent )
+              ? $rparent->ref
+              : undef;
 
-                my $rparent = $point->parent;
-                my $parent
-                  = defined( $rparent )
-                  ? $rparent->ref
-                  : undef;
+            Data::Edit::Struct::failure::input::dest->throw(
+                "point is not an array element" )
+              unless defined $$parent && is_arrayref( $$parent );
 
-                Data::Edit::Struct::failure::input::dest->throw(
-                    "point is not an array element" )
-                  unless defined $$parent && is_arrayref( $$parent );
+            splice( @$$parent, $idx + $offset, $length, @$replace );
+        }
 
-                splice( @$$parent, $idx + $offset, $length, @$replace );
-            }
+        else {
+            Data::Edit::Struct::failure::internal->throw(
+                "_splice: unknown use: $use" );
         }
     }
 }
 
 
-sub _insert ( $dtype, $points, $insert, $anchor, $pad, $offset, $src ) {
+sub _insert {
+
+    my ( $dtype, $points, $insert, $anchor, $pad, $offset, $src ) = @_;
 
     for my $point ( @$points ) {
 
@@ -423,115 +432,121 @@ sub _insert ( $dtype, $points, $insert, $anchor, $pad, $offset, $src ) {
                 "point is neither an array element nor an array ref" );
         }
 
-        for ( $use ) {
+        if ( $use eq 'container' ) {
 
-            when ( 'container' ) {
+            $ref = $point->ref if ! defined $ref;
 
-                $ref //= $point->ref;
+            if ( is_hashref( $$ref ) ) {
 
-                for ( $ref ) {
+                Data::Edit::Struct::failure::input::src->throw(
+                    "insertion into a hash requires an even number of elements\n"
+                ) if @$src % 2;
 
-                    when ( !!is_hashref( $$ref ) ) {
-
-                        Data::Edit::Struct::failure::input::src->throw(
-                            "insertion into a hash requires an even number of elements\n"
-                        ) if @$src % 2;
-
-                        pairmap { ; $$ref->{$a} = $b } @$src;
-                    }
-
-                    when ( !!is_arrayref( $$ref ) ) {
-                        _insert_via_splice( $insert, $anchor, $pad, $ref, 0,
-                            $offset, $src );
-                    }
-
-                    default {
-                        Data::Edit::Struct::failure::input::dest->throw(
-                            "can't insert into a reference of type @{[ ref $$ref]}"
-                        );
-                    }
-                }
+                pairmap { ; $$ref->{$a} = $b } @$src;
             }
 
-            when ( 'element' ) {
-                my $rparent = $point->parent;
-                my $parent
-                  = defined( $rparent )
-                  ? $rparent->ref
-                  : undef;
-
-                Data::Edit::Struct::failure::input::dest->throw(
-                    "point is not an array element" )
-                  unless defined $parent && is_arrayref( $$parent );
-
-                $idx //= ( $attrs // $point->attrs )->{idx};
-
-                _insert_via_splice( $insert, 'index', $pad, $parent, $idx,
+            elsif ( is_arrayref( $$ref ) ) {
+                _insert_via_splice( $insert, $anchor, $pad, $ref, 0,
                     $offset, $src );
             }
+
+            else {
+                Data::Edit::Struct::failure::input::dest->throw(
+                    "can't insert into a reference of type @{[ ref $$ref]}" );
+            }
+        }
+
+        elsif ( $use eq 'element' ) {
+            my $rparent = $point->parent;
+            my $parent
+              = defined( $rparent )
+              ? $rparent->ref
+              : undef;
+
+            Data::Edit::Struct::failure::input::dest->throw(
+                "point is not an array element" )
+              unless defined $parent && is_arrayref( $$parent );
+
+            $idx = ( defined $attrs ? $attrs : $point->attrs )->{idx} if ! defined $idx;
+
+            _insert_via_splice( $insert, 'index', $pad, $parent, $idx,
+                $offset, $src );
+        }
+
+        else {
+            Data::Edit::Struct::failure::internal->throw(
+                "_insert: unknown use: $use" );
         }
     }
 }
 
-sub _insert_via_splice ( $insert, $anchor, $pad, $rdest, $idx, $offset, $src ) {
+sub _insert_via_splice {
+
+    my ( $insert, $anchor, $pad, $rdest, $idx, $offset, $src ) = @_;
 
     my $fididx;
 
-    for ( $anchor ) {
+    if ( $anchor eq 'first' ) {
+        $fididx = 0;
+    }
+    elsif ( $anchor eq 'last' ) {
+        $fididx = $#{ $$rdest };
+    }
+    elsif ( $anchor eq 'index' ) {
+        $fididx = $idx;
+    }
 
-        $fididx = 0 when ( 'first' );
-        $fididx = $$rdest->$#* when ( 'last' );
-        $fididx = $idx when ( 'index' );
-
-        default {
-            Data::Edit::Struct::failure::internal->throw(
-                "unknown insert anchor: $anchor" );
-        }
+    else {
+        Data::Edit::Struct::failure::internal->throw(
+            "unknown insert anchor: $anchor" );
     }
 
     # turn relative index into positive index
     $idx = $offset + $fididx;
 
     # make sure there's enough room.
-    for ( $insert ) {
+    my $maxidx = $#{ $$rdest };
 
-        my $maxidx = $$rdest->$#*;
+    if ( $insert eq 'before' ) {
 
-        when ( 'before' ) {
-
-            if ( $idx < 0 ) {
-                unshift $$rdest->@*, ( $pad ) x ( -$idx );
-                $idx = 0;
-            }
-
-            elsif ( $idx > $maxidx + 1 ) {
-                push $$rdest->@*, ( $pad ) x ( $idx - $maxidx - 1 );
-            }
+        if ( $idx < 0 ) {
+            unshift @{ $$rdest }, ( $pad ) x ( -$idx );
+            $idx = 0;
         }
 
-        when ( 'after' ) {
-
-            if ( $idx < 0 ) {
-                unshift $$rdest->@*, ( $pad ) x ( -$idx - 1 ) if $idx < -1;
-                $idx = 0;
-            }
-
-            elsif ( $idx > $maxidx ) {
-                push $$rdest->@*, ( $pad ) x ( $idx - $maxidx );
-                ++$idx;
-            }
-
-            else {
-                ++$idx;
-            }
-
+        elsif ( $idx > $maxidx + 1 ) {
+            push @{ $$rdest }, ( $pad ) x ( $idx - $maxidx - 1 );
         }
     }
 
+    elsif ( $insert eq 'after' ) {
+
+        if ( $idx < 0 ) {
+            unshift @{ $$rdest }, ( $pad ) x ( -$idx - 1 ) if $idx < -1;
+            $idx = 0;
+        }
+
+        elsif ( $idx > $maxidx ) {
+            push @{ $$rdest }, ( $pad ) x ( $idx - $maxidx );
+            ++$idx;
+        }
+
+        else {
+            ++$idx;
+        }
+
+    }
+    else {
+        Data::Edit::Struct::failure::internal->throw(
+            "_insert_via_splice: unknown insert point: $insert" );
+    }
+
     splice( @$$rdest, $idx, 0, @$src );
 }
 
-sub _delete ( $points, $length ) {
+sub _delete {
+
+    my ( $points, $length ) = @_;
 
     for my $point ( @$points ) {
 
@@ -556,7 +571,7 @@ sub _delete ( $points, $length ) {
 
         }
         else {
-            Data::Edit::Struct::failure::input::internal->throw(
+            Data::Edit::Struct::failure::internal->throw(
                 "point has neither idx nor key attribute" );
         }
 
@@ -564,41 +579,42 @@ sub _delete ( $points, $length ) {
 
 }
 
-sub _replace ( $points, $replace, $src ) {
+sub _replace {
+
+    my ( $points, $replace, $src ) = @_;
 
     for my $point ( @$points ) {
 
         $replace = 'value'
           if $replace eq 'auto';
 
-        for ( $replace ) {
-
-            when ( 'value' ) {
-                $point->ref->$* = $src->$*;
-            }
-
-            when ( 'key' ) {
-
-                my $rparent = $point->parent;
-                my $parent
-                  = defined( $rparent )
-                  ? $rparent->ref
-                  : undef;
-
-                Data::Edit::Struct::failure::input::dest->throw(
-                    "key replacement requires a hash element\n" )
-                  unless is_hashref( $$parent );
-
-                my $old_key = $point->attrs->{key};
-
-                my $new_key = is_ref( $$src ) ? refaddr( $$src ) : $$src;
-
-                $$parent->{$new_key} = delete $$parent->{$old_key};
-            }
+        if ( $replace eq 'value' ) {
+            ${ $point->ref } = ${ $src };
         }
 
-    }
+        elsif ( $replace eq 'key' ) {
 
+            my $rparent = $point->parent;
+            my $parent
+              = defined( $rparent )
+              ? $rparent->ref
+              : undef;
+
+            Data::Edit::Struct::failure::input::dest->throw(
+                "key replacement requires a hash element\n" )
+              unless is_hashref( $$parent );
+
+            my $old_key = $point->attrs->{key};
+
+            my $new_key = is_ref( $$src ) ? refaddr( $$src ) : $$src;
+
+            $$parent->{$new_key} = delete $$parent->{$old_key};
+        }
+        else {
+            Data::Edit::Struct::failure::internal->throw(
+                "_replace: unknown replace type: $replace" );
+        }
+    }
 }
 
 
@@ -622,7 +638,7 @@ Data::Edit::Struct - Edit a Perl structure addressed with a Data::DPath path
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
